package org.gcube.informationsystem.types.impl.entities;

import java.io.StringWriter;

import org.gcube.informationsystem.base.reference.entities.BaseEntity;
import org.gcube.informationsystem.context.reference.entities.Context;
import org.gcube.informationsystem.model.reference.entities.Facet;
import org.gcube.informationsystem.model.reference.entities.Resource;
import org.gcube.informationsystem.model.reference.properties.Header;
import org.gcube.informationsystem.types.impl.TypeDefinitionImpl;
import org.gcube.informationsystem.types.reference.entities.EntityTypeDefinition;
import org.gcube.informationsystem.utils.ISMapper;

import com.fasterxml.jackson.annotation.JsonTypeName;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
@JsonTypeName(value=EntityTypeDefinition.NAME)
public class EntityTypeDefinitionImpl extends TypeDefinitionImpl implements EntityTypeDefinition {

	/**
	 * Generated Serial Version UID
	 */
	private static final long serialVersionUID = 2614315845213942880L;

	// private static final String VERTEX_CLASS_NAME = "V";
	
	public static EntityTypeDefinition getEntityTypeDefinitionInstance(Class<? extends BaseEntity> clz) {
		if(Resource.class.isAssignableFrom(clz)) {
			@SuppressWarnings("unchecked")
			Class<? extends Resource> c = (Class<? extends Resource>) clz;
			return new ResourceTypeDefinitionImpl((Class<? extends Resource>) c);
		} else if(Facet.class.isAssignableFrom(clz)){
			@SuppressWarnings("unchecked")
			Class<? extends Facet> c = (Class<? extends Facet>) clz;
			return new FacetTypeDefinitionImpl(c);
		} 
		return new EntityTypeDefinitionImpl(clz);
	}
	
	
	protected EntityTypeDefinitionImpl() {
		super();
	}
	
	public EntityTypeDefinitionImpl(Class<? extends BaseEntity> clz) {
		super(clz);
		
		if(EntityTypeDefinition.class.isAssignableFrom(clz)){
			@SuppressWarnings("unchecked")
			Class<? extends EntityTypeDefinition> c = (Class<? extends EntityTypeDefinition>) clz;
			this.superClasses = retrieveSuperClasses(c, EntityTypeDefinition.class, BaseEntity.NAME);
		} else if(Context.class.isAssignableFrom(clz)){
			@SuppressWarnings("unchecked")
			Class<? extends Context> c = (Class<? extends Context>) clz;
			this.superClasses = retrieveSuperClasses(c, Context.class, BaseEntity.NAME);
		} else if(BaseEntity.class.isAssignableFrom(clz)){
			this.superClasses = retrieveSuperClasses(clz, BaseEntity.class, null);
		} else {
			throw new RuntimeException("Type Hierachy Error for class " + clz.getSimpleName());
		}
	}

	
	/* 
	 * Java does not support multiple inheritance. 
	 * TypeDefinitionImpl is the superclass so that this class does not inherits the methods and field of BaseRelationImpl
	 * We need to copy them. 
	 */
	
	protected Header header;
	
	@Override
	public Header getHeader() {
		return header;
	}
	
	@Override
	public void setHeader(Header header){
		this.header = header;
	}
	
	@Override
	public String toString(){
		StringWriter stringWriter = new StringWriter();
		try {
			ISMapper.marshal(this, stringWriter);
			return stringWriter.toString();
		}catch(Exception e){
			try {
				ISMapper.marshal(this.header, stringWriter);
				return stringWriter.toString();
			} catch(Exception e1){
				return super.toString();
			}
		}
	}

}
