package org.gcube.informationsystem.types.impl.relations;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import org.gcube.informationsystem.base.reference.entities.BaseEntity;
import org.gcube.informationsystem.base.reference.relations.BaseRelation;
import org.gcube.informationsystem.context.reference.relations.IsParentOf;
import org.gcube.informationsystem.model.reference.entities.Facet;
import org.gcube.informationsystem.model.reference.entities.Resource;
import org.gcube.informationsystem.model.reference.relations.ConsistsOf;
import org.gcube.informationsystem.model.reference.relations.IsRelatedTo;
import org.gcube.informationsystem.types.impl.TypeDefinitionImpl;
import org.gcube.informationsystem.types.impl.entities.EntityTypeDefinitionImpl;
import org.gcube.informationsystem.types.reference.entities.EntityTypeDefinition;
import org.gcube.informationsystem.types.reference.relations.RelationTypeDefinition;

import com.fasterxml.jackson.annotation.JsonTypeName;

@JsonTypeName(value = RelationTypeDefinition.NAME)
public class RelationTypeDefinitionImpl<S extends EntityTypeDefinition, T extends EntityTypeDefinition>
		extends TypeDefinitionImpl implements RelationTypeDefinition<S,T> {
	
	/**
	 * Generated Serial Version UID
	 */
	private static final long serialVersionUID = 2221831081869571296L;
	
	protected RelationTypeDefinitionImpl() {
		super();
	}
	
	public static RelationTypeDefinition<?,?> getRelationTypeDefinitionInstance(
			Class<? extends BaseRelation<?,?>> clz) {
		if(IsRelatedTo.class.isAssignableFrom(clz)) {
			@SuppressWarnings("unchecked")
			Class<? extends IsRelatedTo<? extends Resource,? extends Resource>> c = (Class<? extends IsRelatedTo<? extends Resource,? extends Resource>>) clz;
			return new IsRelatedToTypeDefinitionImpl(c);
		} else if(ConsistsOf.class.isAssignableFrom(clz)) {
			@SuppressWarnings("unchecked")
			Class<? extends ConsistsOf<? extends Resource,? extends Facet>> c = (Class<? extends ConsistsOf<? extends Resource,? extends Facet>>) clz;
			return new ConsistsOfTypeDefinitionImpl(c);
		}
		return new RelationTypeDefinitionImpl<EntityTypeDefinition,EntityTypeDefinition>(clz);
	}
	
	public RelationTypeDefinitionImpl(Class<? extends BaseRelation<?,?>> clz) {
		super(clz);
		
		if(RelationTypeDefinition.class.isAssignableFrom(clz)) {
			@SuppressWarnings({"unchecked"})
			Class<RelationTypeDefinition<?,?>> c = (Class<RelationTypeDefinition<?,?>>) clz;
			this.superClasses = retrieveSuperClasses(c, RelationTypeDefinition.class, BaseRelation.NAME);
		} else if(IsParentOf.class.isAssignableFrom(clz)) {
			@SuppressWarnings("unchecked")
			Class<? extends IsParentOf> c = (Class<? extends IsParentOf>) clz;
			this.superClasses = retrieveSuperClasses(c, IsParentOf.class, BaseRelation.NAME);
		} else if(BaseRelation.class.isAssignableFrom(clz)) {
			this.superClasses = retrieveSuperClasses(clz, BaseRelation.class, null);
		} else {
			throw new RuntimeException("Type Hierachy Error");
		}
		
		discoverSourceAndTarget(clz);
		
	}
	
	@SuppressWarnings("unchecked")
	protected Type[] getParametersFromSuperClasses(Class<? extends BaseRelation<?,?>> clz) {
		for(Type t : clz.getGenericInterfaces()) {
			if(t instanceof ParameterizedType) {
				if(BaseRelation.class.isAssignableFrom((Class<?>) ((ParameterizedType) t).getRawType())) {
					return ((ParameterizedType) t).getActualTypeArguments();
				}
			} else if(BaseRelation.class.isAssignableFrom((Class<?>) t)) {
				return getParametersFromSuperClasses((Class<? extends BaseRelation<?,?>>) t);
			}
		}
		throw new RuntimeException("Unable to find Generic Parameters From SuperClasses to set source and target");
	}
	
	@SuppressWarnings("unchecked")
	private void discoverSourceAndTarget(Class<? extends BaseRelation<?,?>> clz) {
		Type[] typeParameters = clz.getTypeParameters();
		
		Class<? extends BaseEntity> sourceClass;
		Class<? extends BaseEntity> targetClass;
		if(typeParameters.length == 0) {
			typeParameters = getParametersFromSuperClasses(clz);
			sourceClass = (Class<? extends BaseEntity>) typeParameters[0];
			targetClass = (Class<? extends BaseEntity>) typeParameters[1];
		} else {
			sourceClass = (Class<? extends BaseEntity>) getGenericClass(typeParameters[0]);
			targetClass = (Class<? extends BaseEntity>) getGenericClass(typeParameters[1]);
		}
		
		this.source = (S) new EntityTypeDefinitionImpl(sourceClass);
		this.target = (T) new EntityTypeDefinitionImpl(targetClass);
		
	}
	
	/* 
	 * Java does not support class multiple inheritance. 
	 * TypeDefinitionImpl is the superclass so that this class does not inherits the methods and field of BaseRelationImpl
	 * We need to copy them. 
	 */
	
	protected S source;
	protected T target;
	
	@Override
	public S getSource() {
		return source;
	}
	
	@Override
	public void setSource(S source) {
		this.source = source;
	}
	
	@Override
	public T getTarget() {
		return target;
	}
	
	@Override
	public void setTarget(T target) {
		this.target = target;
	}
	
}
