package org.gcube.informationsystem.types;

import org.gcube.informationsystem.base.reference.entities.BaseEntity;
import org.gcube.informationsystem.base.reference.properties.BaseProperty;
import org.gcube.informationsystem.context.reference.entities.Context;
import org.gcube.informationsystem.context.reference.relations.IsParentOf;
import org.gcube.informationsystem.model.impl.properties.PropagationConstraintImpl;
import org.gcube.informationsystem.model.reference.entities.Entity;
import org.gcube.informationsystem.model.reference.entities.Facet;
import org.gcube.informationsystem.model.reference.entities.Resource;
import org.gcube.informationsystem.model.reference.properties.Encrypted;
import org.gcube.informationsystem.model.reference.properties.Header;
import org.gcube.informationsystem.model.reference.properties.PropagationConstraint;
import org.gcube.informationsystem.model.reference.properties.PropagationConstraint.AddConstraint;
import org.gcube.informationsystem.model.reference.properties.PropagationConstraint.RemoveConstraint;
import org.gcube.informationsystem.model.reference.properties.Property;
import org.gcube.informationsystem.model.reference.relations.ConsistsOf;
import org.gcube.informationsystem.model.reference.relations.IsRelatedTo;
import org.gcube.informationsystem.model.reference.relations.Relation;
import org.gcube.informationsystem.types.reference.entities.EntityTypeDefinition;
import org.gcube.informationsystem.types.reference.entities.FacetTypeDefinition;
import org.gcube.informationsystem.types.reference.entities.ResourceTypeDefinition;
import org.gcube.informationsystem.types.reference.properties.PropertyDefinition;
import org.gcube.informationsystem.types.reference.properties.PropertyTypeDefinition;
import org.gcube.informationsystem.types.reference.properties.ResourceEntryDefinition;
import org.gcube.informationsystem.types.reference.relations.ConsistsOfTypeDefinition;
import org.gcube.informationsystem.types.reference.relations.IsRelatedToTypeDefinition;
import org.gcube.informationsystem.types.reference.relations.RelationTypeDefinition;
import org.gcube.informationsystem.utils.ISMapper;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

public class SerializationTest {

	private static Logger logger = LoggerFactory.getLogger(SerializationTest.class);
	
	@Test
	public void serialize() throws Exception{
		TypeBinder.serializeType(EntityTest.class);
	}
	
	@Test
	public void makeTypeDefinition() throws Exception{
		EntityTypeDefinition resourceTypeDefinitionSelf = (EntityTypeDefinition) TypeBinder.createTypeDefinition(ResourceTypeDefinition.class);
		logger.info(ISMapper.marshal(resourceTypeDefinitionSelf));
	}
	
	@Test
	public void testPropertyTypeDefinition() throws Exception{
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<BaseProperty> basePropertyTypeDefinition = (PropertyTypeDefinition<BaseProperty>) TypeBinder.createTypeDefinition(BaseProperty.class);
		logger.info(ISMapper.marshal(basePropertyTypeDefinition));
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<Property> propertyTypeDefinition = (PropertyTypeDefinition<Property>) TypeBinder.createTypeDefinition(Property.class);
		logger.info(ISMapper.marshal(propertyTypeDefinition));
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<Header> headerTypeDefinition = (PropertyTypeDefinition<Header>) TypeBinder.createTypeDefinition(Header.class);
		logger.info(ISMapper.marshal(headerTypeDefinition));
		
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<PropagationConstraint> propagationConstraintDefinition = (PropertyTypeDefinition<PropagationConstraint>) TypeBinder.createTypeDefinition(PropagationConstraint.class);
		logger.info(ISMapper.marshal(propagationConstraintDefinition));
		
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<Encrypted> encryptedTypeDefinition = (PropertyTypeDefinition<Encrypted>) TypeBinder.createTypeDefinition(Encrypted.class);
		logger.info(ISMapper.marshal(encryptedTypeDefinition));
		
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<PropertyDefinition> propertyDefinition = (PropertyTypeDefinition<PropertyDefinition>) TypeBinder.createTypeDefinition(PropertyDefinition.class);
		logger.info(ISMapper.marshal(propertyDefinition));
		
		@SuppressWarnings("unchecked")
		PropertyTypeDefinition<ResourceEntryDefinition> resourceEntryDefinition = (PropertyTypeDefinition<ResourceEntryDefinition>) TypeBinder.createTypeDefinition(ResourceEntryDefinition.class);
		logger.info(ISMapper.marshal(resourceEntryDefinition));
	}
	
	@Test
	public void testEntityTypeDefinition() throws Exception{
		EntityTypeDefinition baseEntityTypeDefinition = (EntityTypeDefinition) TypeBinder.createTypeDefinition(BaseEntity.class);
		logger.info(ISMapper.marshal(baseEntityTypeDefinition));
		
		EntityTypeDefinition entityTypeDefinition = (EntityTypeDefinition) TypeBinder.createTypeDefinition(Entity.class);
		logger.info(ISMapper.marshal(entityTypeDefinition));
		
		ResourceTypeDefinition resourceTypeDefinition = (ResourceTypeDefinition) TypeBinder.createTypeDefinition(Resource.class);
		logger.info(ISMapper.marshal(resourceTypeDefinition));
		
		FacetTypeDefinition facetTypeDefinition = (FacetTypeDefinition) TypeBinder.createTypeDefinition(Facet.class);
		logger.info(ISMapper.marshal(facetTypeDefinition));
		
		EntityTypeDefinition contextTypeDefinition = (EntityTypeDefinition) TypeBinder.createTypeDefinition(Context.class);
		logger.info(ISMapper.marshal(contextTypeDefinition));
		
		EntityTypeDefinition entityTypeDefinitionSelf = (EntityTypeDefinition) TypeBinder.createTypeDefinition(EntityTypeDefinition.class);
		logger.info(ISMapper.marshal(entityTypeDefinitionSelf));
		
		EntityTypeDefinition resourceTypeDefinitionSelf = (EntityTypeDefinition) TypeBinder.createTypeDefinition(ResourceTypeDefinition.class);
		logger.info(ISMapper.marshal(resourceTypeDefinitionSelf));
		
		EntityTypeDefinition facetTypeDefinitionSelf = (EntityTypeDefinition) TypeBinder.createTypeDefinition(FacetTypeDefinition.class);
		logger.info(ISMapper.marshal(facetTypeDefinitionSelf));
	}
	
	@Test
	public void testRelationTypeDefinition() throws Exception{
		@SuppressWarnings("unchecked")
		RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition> relationTypeDefinition = (RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition>) TypeBinder.createTypeDefinition(Relation.class);
		logger.info(ISMapper.marshal(relationTypeDefinition));
		
		IsRelatedToTypeDefinition isRelatedToTypeDefinition = (IsRelatedToTypeDefinition) TypeBinder.createTypeDefinition(IsRelatedTo.class);
		logger.info(ISMapper.marshal(isRelatedToTypeDefinition));
		
		ConsistsOfTypeDefinition consistsOfTypeDefinition = (ConsistsOfTypeDefinition) TypeBinder.createTypeDefinition(ConsistsOf.class);
		logger.info(ISMapper.marshal(consistsOfTypeDefinition));
		
		@SuppressWarnings("unchecked")
		RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition> isParentOfTypeDefinition = (RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition>) TypeBinder.createTypeDefinition(IsParentOf.class);
		logger.info(ISMapper.marshal(isParentOfTypeDefinition));
		
		@SuppressWarnings("unchecked")
		RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition> rtdSelf = (RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition>) TypeBinder.createTypeDefinition(RelationTypeDefinition.class);
		logger.info(ISMapper.marshal(rtdSelf));

		@SuppressWarnings("unchecked")
		RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition> isRelatedToTypeDefinitionSelf = (RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition>) TypeBinder.createTypeDefinition(IsRelatedToTypeDefinition.class);
		logger.info(ISMapper.marshal(isRelatedToTypeDefinitionSelf));
		
		@SuppressWarnings("unchecked")
		RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition> consistsOfTypeDefinitionSelf = (RelationTypeDefinition<EntityTypeDefinition,EntityTypeDefinition>) TypeBinder.createTypeDefinition(ConsistsOfTypeDefinition.class);
		logger.info(ISMapper.marshal(consistsOfTypeDefinitionSelf));
	}
	
	@Test
	public void testGetEnumcostants(){
		Class<?> clz = PropagationConstraint.RemoveConstraint.class;
		
		Object[] constants = clz.getEnumConstants();
		for(Object constant : constants){
			logger.trace("{}", constant.toString());
		}
		
	}
	
	@Test
	public void testPropagationConstraint() throws Exception {
		PropagationConstraint propagationConstraint = new PropagationConstraintImpl();
		propagationConstraint.setAddConstraint(AddConstraint.propagate);
		propagationConstraint.setRemoveConstraint(RemoveConstraint.cascadeWhenOrphan);
		
		ObjectMapper mapper = new ObjectMapper();
		String pg = mapper.writeValueAsString(propagationConstraint);
		
		PropagationConstraint pgUnm = mapper.readValue(pg, PropagationConstraint.class);
		
		logger.debug("{}", pgUnm);
	}
	
	@Test
	public void testTypeSerialization() throws Exception {
		String serialized = TypeBinder.serializeType(Header.class);
		logger.info(serialized);
	}
	
}
